@@ -22,6 +22,22 @@ import matplotlib.ticker as mticker
 from scipy.ndimage.filters import minimum_filter, maximum_filter
 
 
+def get_mplstyle(name):
+    """
+    Return matplotlib style filepath.
+
+    Args:
+        name (str): mplstyle name.
+    """
+    
+    file = pkg_resources.resource_filename(
+            'nmc_met_graphics', "resources/mplstyle/{}.mplstyle".format(name))
+    if os.path.isfile(file):
+        return file
+    else:
+        raise ValueError('Cannot find the {} sytle file.'.format(file))
+
+
 def add_gridlines(ax, draw_labels=True, linewidth=2, color='gray', alpha=0.5,
                   linestyle='--', xlabels_top=False, ylabels_right=False,
                   xlabel_style={'size': 16}, ylabel_style={'size': 16},
@@ -422,7 +438,7 @@ def add_mslp_label(ax, proj_ccrs, mslp, lat, lon):
         return np.nonzero(mat == mn), np.nonzero(mat == mx)
     
     #Determine an appropriate window given the lat/lon grid resolution
-    res = lat[1] - lat[0]
+    res = np.abs(lat[1] - lat[0])
     nwindow = int(9.5 / res)
     mslp = np.ma.masked_invalid(mslp)
     local_min, local_max = _extrema(mslp, mode='wrap', window=nwindow)
@@ -473,3 +489,4 @@ def add_mslp_label(ax, proj_ccrs, mslp, lat, lon):
                        mpatheffects.SimpleLineShadow(),mpatheffects.Normal()])
                 xyplotted.append((x,y))
 
+
